# AUTOGENERATED! DO NOT EDIT! File to edit: xlsx.ipynb (unless otherwise specified).

__all__ = ['to_excel', 'column_format', 'default_formats', 'column_formats', 'format_sheet']

# Internal Cell
import openpyxl
from openpyxl.styles import Alignment
from openpyxl.worksheet.table import Table, TableStyleInfo
import re
from pathlib import Path
import toolz.curried as tz
from fnmatch import fnmatch

# Internal Cell
from jedi_toolz import show
from datetime import date
from dateutil.relativedelta import relativedelta as delta
from random import choice, randint, random

names = (
    "Joe Nancy Harry Gus David Max Matt Nicole Tracy Stacy "
    "Kristi Kelly Mike James"
)
texts = (
    "Univeristy Franklin End_Game Tower New_York_Mets Last_Dance Movies Donald_Trump "
    "Nation_Captial COVID-19 Honesty Some_Really_Long_Text The_Lord_of_the_Rings"
)
flag = lambda: True if randint(1, 10) < 10 else False
text = lambda: choice([val.replace("_", " ") for val in texts.split()]) if flag() else None
name = lambda: choice(names.split())
dt = lambda: date.today() + delta(days=randint(-30, 30)) if flag() else None
amt = lambda: randint(150_000, 350_000) * random()

# Internal Cell
gen_data = lambda n=100: [
    {
        "Order Date": dt(),
        "Close Date": dt(),
        "Name": name(),
        "Dept": text(),
        "Balance": amt(),
        "Loan": amt(),
    }
    for row in range(n)
]
show(gen_data())

# Cell
def to_excel(data, file, sheet_name, over_write=True):
    if not over_write and file.exists():
        wb = openpyxl.load_workbook(file)
        ws = wb.create_sheet(sheet_name)
    else:
        wb = openpyxl.Workbook()
        ws = wb.active
        ws.title = sheet_name
    for row_num, row in enumerate(data):
        if row_num == 0:
            ws.append([k for k in row])
        ws.append(v for v in row.values())
    wb.save(str(file))
    wb.close()
    return file

# Internal Cell
def make_test_file():
    file = Path.cwd() / "test.xlsx"
    sheet_name = "Test 1"
    over_write = True
    data = gen_data(100)
    return to_excel(data, file, sheet_name, over_write)

# Cell
def column_format(number_format="", width=15, vertical="top", horizontal="left", wrap_text=True, **kwargs):
    return {
        "number_format": number_format,
        "alignment": Alignment(vertical=vertical, horizontal=horizontal, wrap_text=wrap_text, **kwargs),
        "width": width,
    }

# Cell
def default_formats(file, sheet_name, rows_to_review=100):
    wb = openpyxl.load_workbook(str(file), read_only=True)
    ws = wb[sheet_name]
    rows = ws.rows
    header = next(rows)
    names = [cell.value for cell in header]
    letters = [cell.column_letter for cell in header]
    return tz.pipe(
        rows,
        tz.take(rows_to_review),
        tz.map(lambda row: dict(zip(names, row))),
        tz.map(tz.valmap(tz.flip(getattr, "data_type"))),
        tz.merge_with(list),
        tz.valmap(tz.frequencies),
        lambda row: {
            k: (
                column_format("YYYY-MM-DD", horizontal="right") if "d" in v else
                column_format() if "s" in v else
                column_format("#,##0.00", horizontal="right")
            )
            for k, v in row.items()
        },
    )

# Cell
def column_formats(file, sheet_name, patterns=None):
    defaults = default_formats(file, sheet_name)
    if patterns is None:
        return defaults

    results = {}
    for col in defaults:
        for pat, fmt in patterns.items():
            if fnmatch(col, pat) and col not in results:
                results[col] = fmt
        if col not in results:
            results[col] = defaults[col]
    return results

# Cell
def format_sheet(file, sheet_name, format_patterns=None, table_name=None, freeze_panes="A2"):
    # Load file
    wb = openpyxl.load_workbook(file)
    ws = wb[sheet_name]

    # Get column names and calc formats
    rows = ws.rows
    header = next(rows)
    column_names = [cell.value for cell in next(rows)]
    formats = column_formats(file, sheet_name, format_patterns)

    # Loop by column and format column
    for col in ws.iter_cols():
        header = col[0]
        name = header.value
        letter = header.column_letter

        # Set width first so that text wrap works
        ws.column_dimensions[letter].width = formats[name]["width"]

        # Add alignment and number format
        for cell in col:
            cell.alignment = formats[name]["alignment"]
            cell.number_format = formats[name]["number_format"]

    # Freeze panes
    ws.freeze_panes = ws[freeze_panes]

    # Add table
    table_name = table_name or sheet_name
    table_name = table_name.replace(" ", "_")
    table = Table(displayName=table_name, ref=ws.dimensions)
    style = TableStyleInfo(name="TableStyleLight1", showFirstColumn=False, showColumnStripes=False, showRowStripes=True)
    table.tableStyleInfo = style
    ws.add_table(table)

    # Save file
    wb.save(str(file))
    wb.close()


# AUTOGENERATED! DO NOT EDIT! File to edit: show.ipynb (unless otherwise specified).

__all__ = ['transpose', 'wrap_row', 'wrap_table', 'show']

# Internal Cell
try:
    import pandas
    has_pandas = lambda: True
except ModuleNotFoundError:
    has_pandas = lambda: False

from tabulate import tabulate
from textwrap import fill
import toolz.curried as tz
import itertools as it
from decorator import decorator

# Internal Cell
@decorator
def handle_pandas(func, *args, **kwargs):
    orig = [arg for arg in args]
    data = orig[0]
    if has_pandas() and isinstance(data, pandas.DataFrame):
        data = data.to_dict("records")
    updated = [data if num == 0 else arg for num, arg in enumerate(orig)]
    return func(*updated, **kwargs)

# Internal Cell
@handle_pandas
def text_table(data, print_out=True, **tabulate_args):
    defaults = {
        "tablefmt": "fancy_grid",
        "floatfmt": ",.2f",
        "headers": "keys"
    }
    tabulate_args = {**defaults, **tabulate_args}
    tab = tz.partial(tabulate, **tabulate_args)
    if print_out:
        print(tab(data))
    else:
        return tab(data)

# Internal Cell
@handle_pandas
def head(data, limit=100):
    return list(tz.take(limit, data))

# Cell
@handle_pandas
def transpose(data):
    count = it.count(1)
    row_num = lambda: next(count)
    header = lambda d: list(d.keys())
    values = lambda d: list(zip(*d.values()))
    combine = lambda d: [dict(zip(header(d), row)) for row in values(d)]
    return tz.pipe(
        data,
        tz.map(lambda row: list(zip(*row.items()))),
        tz.map(lambda row: dict(zip(["column", f"row {row_num()}"], row))),
        tz.merge,
        combine,
    )

# Cell
def wrap_row(row, col_width):
    result = {}
    for k, v in row.items():
        new_k = fill(str(k), col_width)
        new_v = fill(str(v), col_width)
        result[new_k] = new_v
    return result

# Cell
@handle_pandas
def wrap_table(data, col_width):
    """Takes a list of dicts and wraps the keys and values by the
    specified col_width."""
    return [wrap_row(row, col_width) for row in data]

# Internal Cell
def text_width(text):
    return max(len(line) for line in text.splitlines())

# Internal Cell
@handle_pandas
def get_text(data, limit, vert, col_width):
    return tz.pipe(
        head(data, limit),
        lambda table: transpose(table) if vert else table,
        tz.partial(wrap_table, col_width=col_width),
        tz.partial(text_table, print_out=False),
    )

# Cell
@tz.curry
@handle_pandas
def show(data, limit=30, vert=False, col_width=15, table_width=80):
    orig = get_text(data, limit, vert, col_width)
    if text_width(orig) <= table_width:
        print(orig)
        return None
    else:
        good = get_text(data, limit=1, vert=True, col_width=col_width)
        msg = (
            "Showing 1 row vertically exceeds table_width. "
            "Please adjust table_width or col_width."
        )
        assert text_width(good) <= table_width, msg

    for n in range(2, len(data)):
        new = get_text(data, n, True, col_width)
        if text_width(new) <= table_width:
            good = new
        else:
            print(good)
            return None


# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"__jedi_toolz_var__": "config.ipynb",
         "__jedi_toolz_path__": "config.ipynb",
         "__jedi_toolz_path_method__": "config.ipynb",
         "set_path": "config.ipynb",
         "as_dict": "config.ipynb",
         "as_records": "config.ipynb",
         "select": "config.ipynb",
         "example_ini": "config.ipynb",
         "__tables__": "domo.ipynb",
         "connect": "domo.ipynb",
         "tables": "domo.ipynb",
         "get_id": "domo.ipynb",
         "query": "domo.ipynb",
         "handle_pandas": "show.ipynb",
         "text_table": "show.ipynb",
         "head": "show.ipynb",
         "transpose": "show.ipynb",
         "wrap_row": "show.ipynb",
         "wrap_table": "show.ipynb",
         "text_width": "show.ipynb",
         "get_text": "show.ipynb",
         "show": "show.ipynb",
         "names": "xlsx.ipynb",
         "texts": "xlsx.ipynb",
         "flag": "xlsx.ipynb",
         "text": "xlsx.ipynb",
         "name": "xlsx.ipynb",
         "dt": "xlsx.ipynb",
         "amt": "xlsx.ipynb",
         "gen_data": "xlsx.ipynb",
         "to_excel": "xlsx.ipynb",
         "make_test_file": "xlsx.ipynb",
         "column_format": "xlsx.ipynb",
         "default_formats": "xlsx.ipynb",
         "column_formats": "xlsx.ipynb",
         "format_sheet": "xlsx.ipynb"}

modules = ["config.py",
           "domo.py",
           "show.py",
           "xlsx.py"]

doc_url = "https://JediHero.github.io/jedi_toolz/"

git_url = "https://github.com/JediHero/jedi_toolz/tree/master/"

def custom_doc_links(name): return None
